@@ -30,38 +30,51 @@ bedToGR <- function(x, skip.name = TRUE) {
 #' Additional specific columns are:\cr
 #' - width (if not set, width is set to 1 for all reads)\cr
 #' Additional columns will be assigned as meta columns
+#' @param keep.extra.columns logical, default TRUE, keep meta cols.
 #' @inheritParams import.ofst
 #' @return GRanges object
 #' @importFrom S4Vectors new2
 #' @keywords internal
-getGRanges <- function(df, seqinfo = NULL) {
+getGRanges <- function(df, keep.extra.columns = TRUE, seqinfo = NULL) {
+  stopifnot(is(df, "data.frame"))
+  if (!isTRUEorFALSE(keep.extra.columns))
+    stop("'keep.extra.columns' must be TRUE or FALSE")
   if (!all(c("seqnames", "start", "strand") %in% colnames(df)))
-    stop("df must at minimum have 4 columns named: seqnames, start, width and strand")
+    stop("df must at minimum have 3 columns named: seqnames, start, strand")
+  if (!is(df, "data.table")) setDT(df)
 
   widths <- if ("width" %in% colnames(df)) {
     df$width
-  } else rep.int(1L, nrow(df))
+  } else {
+    if ("end" %in% colnames(df)) {
+      df$end - df$start  + 1L
+    } else rep.int(1L, nrow(df))
+  }
+
   ranges <- new2("IRanges", start = df$start,
                  width = widths,
                  NAMES = df$NAMES,
                  elementMetadata = NULL,
                  check = FALSE)
   if (is.null(levels(df$seqnames))) {
-    df$seqnames <- factor(df$seqnames, levels = unique(df$seqnames))
+    df[, seqnames := factor(seqnames, levels = unique(seqnames))]
+  }
+  if (is.null(levels(df$strand))) {
+    df[, strand := factor(strand, levels = c("+", "-", "*"))]
   }
   if (!is.null(seqinfo)) {
     stopifnot(is(seqinfo, "Seqinfo"))
   } else seqinfo <- Seqinfo(levels(df$seqnames))
-  df$NAMES <- NULL
-  mcols_hits <- !(colnames(df) %in% c("seqnames", "start", "strand", "width"))
-  has_mcols <- any(mcols_hits)
-  if (!has_mcols){
-    mcols <- NULL
-  } else {
-    mcols <- df[,which(mcols_hits)]
-    if (sum(mcols_hits) == 1) {
-      mcols <- data.frame(mcols)
-      names(mcols) <- names(df)[mcols_hits]
+
+  if (!is.null(df$NAMES)) df$NAMES <- NULL
+
+
+  mcols <- NULL
+  if (keep.extra.columns) {
+    mcols_hits <- !(colnames(df) %in% c("seqnames", "start", "strand", "width", "end"))
+    has_mcols <- any(mcols_hits)
+    if (has_mcols) {
+      mcols <- df[, which(mcols_hits), with = FALSE]
     }
   }
   mcols <- S4Vectors:::normarg_mcols(mcols, "GRanges", nrow(df))
@@ -70,6 +83,30 @@ getGRanges <- function(df, seqinfo = NULL) {
        elementMetadata = mcols, seqinfo = seqinfo, check = FALSE)
 }
 
+#' Faster version (also less safe) of makeGRangesFromDataFrame
+#'
+#' @inheritParams getGRanges
+#' @return GRanges object
+#' @export
+#' @examples
+#' df <- data.frame(start = rep(1L, 1e5), end = 10L, strand = "+", seqnames = "1")
+#'
+#' system.time(res <- makeGRangesFromDataFrame(df))
+#' system.time(res_fast <- makeGRangesFromDataFrameFast(df))
+#' identical(res, res_fast)
+#'
+#' # Use width instead of end, does not work in original
+#' df2 <- data.frame(start = rep(1L, 1e5), width = 10L, strand = "+", seqnames = "1")
+#' system.time(makeGRangesFromDataFrameFast(df2))
+#' df_small <- data.frame(start = 1L, end = 10L, strand = "+", seqnames = "1")
+#' system.time(res <- makeGRangesFromDataFrame(df_small))
+#' system.time(res_fast <- makeGRangesFromDataFrameFast(df_small))
+#' identical(res, res_fast)
+makeGRangesFromDataFrameFast <- function(df, keep.extra.columns = TRUE,
+                                         seqinfo = NULL) {
+  return(getGRanges(df, keep.extra.columns, seqinfo))
+}
+
 #' Internal GAlignments loader from fst data.frame
 #' @param df a data.frame/data.table with columns minimum 4 columns:
 #' seqnames, start ("pos" in final GA object), cigar and strand.\cr
@@ -103,7 +140,7 @@ getGAlignments <- function(df, seqinfo = NULL) {
       names(mcols) <- names(df)[5]
     }
   }
-  if (!is(df$strand, "factor") && identical(levels(df$strand), c("+", "-", "*"))){
+  if (!is(df$strand, "factor") | identical(levels(df$strand), c("+", "-", "*"))){
     df[, strand := factor(strand, levels = c("+", "-", "*"))]
   }
 
@@ -124,8 +161,8 @@ getGAlignments <- function(df, seqinfo = NULL) {
 #' @keywords internal
 getGAlignmentsPairs <- function(df, strandMode = 0, seqinfo = NULL) {
   if (nrow(df) == 0) {
-    return(GenomicAlignments::GAlignmentPairs(first = GAlignments(),
-                                              last = GAlignments(),
+    return(GenomicAlignments::GAlignmentPairs(first = GAlignments(seqinfo = seqinfo),
+                                              last = GAlignments(seqinfo = seqinfo),
                                               strandMode = strandMode))
   }
   if (is.null(levels(df$seqnames))) {
@@ -136,6 +173,12 @@ getGAlignmentsPairs <- function(df, strandMode = 0, seqinfo = NULL) {
   } else seqinfo <- Seqinfo(levels(df$seqnames))
   names <- df$NAMES
   if (!is.null(df$NAMES)) df$NAMES <- NULL
+
+  isProperPair <- df$isProperPair
+  if (!is.null(df$isProperPair)) {
+    df$isProperPair <- NULL
+  } else isProperPair <- rep(TRUE, nrow(df))
+
   if (ncol(df) == 6){
     mcols <- NULL
   } else {
@@ -162,7 +205,7 @@ getGAlignmentsPairs <- function(df, strandMode = 0, seqinfo = NULL) {
              cigar = as.character(df$cigar2), strand = Rle(factor(strand2, levels)),
              seqinfo = seqinfo, check = FALSE,
              elementMetadata = DataFrame(data.frame(matrix(nrow = nrow(df), ncol = 0)))),
-       isProperPair = rep(TRUE, nrow(df)), strandMode = as.integer(strandMode),
+       isProperPair = isProperPair, strandMode = as.integer(strandMode),
        elementMetadata = mcols, check = FALSE)
 }
 
@@ -372,9 +415,9 @@ combn.pairs <- function(x) {
 #' Read RDS or QS format file
 #'
 #' @param file path to file with "rds" or "qs" file extension
-#' @param nthread numeric, number of threads for qs::qread
+#' @param nthread numeric, number of threads for qs2::qs_read
 #' @return R object loaded from file
-#' @importFrom qs qread
+#' @importFrom qs2 qs_read
 #' @export
 #' @examples
 #' df <- ORFik::ORFik.template.experiment()
@@ -385,16 +428,16 @@ read_RDSQS <- function(file, nthread = 5) {
   stopifnot(format %in% c("qs", "rds", "covqs", "covrds"))
   if (format %in% c("rds", "covrds")) {
     readRDS(file)
-  } else qs::qread(file, nthread = nthread)
+  } else qs2::qs_read(file, nthread = nthread)
 }
 
 #' Read RDS or QS format file
 #'
 #' @param object the object to save
 #' @param file path to file with "rds" or "qs" file extension
-#' @param nthread numeric, number of threads for qs::qread
+#' @param nthread numeric, number of threads for qs2::qs_save
 #' @return R object loaded from file
-#' @importFrom qs qread
+#' @importFrom qs2 qs_save
 #' @export
 #' @examples
 #' path <- tempfile(fileext = ".qs")
@@ -412,7 +455,7 @@ save_RDSQS <- function(object, file, nthread = 5) {
   stopifnot(format %in% c("qs", "rds", "covqs", "covrds"))
   if (format %in% c("rds", "covrds")) {
     saveRDS(object, file)
-  } else qs::qsave(object, file, nthread = nthread)
+  } else qs2::qs_save(object, file, nthread = nthread)
 }
 
 #' A fast ftp directory check
@@ -493,6 +536,12 @@ exists.ftp.file.fast <- function(url, report.error = FALSE) {
   return(FALSE)
 }
 
+local_DTthreads <- function(n = 1, env = parent.frame()) {
+  old <- data.table::getDTthreads()
+  withr::defer(data.table::setDTthreads(old), envir = env)
+  data.table::setDTthreads(n)
+}
+
 #' Detects the mounted drive based on a mounted path
 #' @param ref_path = path.expand(config()["ref"])
 #' @return character, name of FileSystem drive of mounted path,
